"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var redux_1 = require("redux");
var rxjs_1 = require("rxjs");
var constants_1 = require("../../helpers/constants");
var isEvent_1 = require("../../helpers/is/isEvent/isEvent");
var isSubscribable_1 = require("../../helpers/is/isSubscribable/isSubscribable");
var getRootReducer_1 = require("./getRootReducer");
/**
 * @private
 */
var defaultEnableDevTools = function () { return process.env.NODE_ENV !== 'production'; };
var getReduxEnhancer = function (config) {
    if (!config) {
        return redux_1.compose;
    }
    var _a = config.enableDevTools, enableDevTools = _a === void 0 ? defaultEnableDevTools : _a;
    if (enableDevTools() &&
        typeof window === 'object' && // tslint:disable-line strict-type-predicates
        typeof window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ === 'function') {
        return window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__(config);
    }
    return redux_1.compose;
};
/**
 * @private
 */
var updateMeta = function (name, event) {
    event.meta = event.meta || {};
    event.meta[constants_1.SOURCE] = (event.meta[constants_1.SOURCE] || []).concat(name);
};
/**
 * @private
 * @internal
 */
exports.getStore = function (reducers, initialState, middlewares, devtools) {
    var initializing = true;
    var queue = [];
    var completed = false;
    var store = redux_1.createStore(getRootReducer_1.getRootReducer(reducers, initialState), initialState, getReduxEnhancer(devtools)(redux_1.applyMiddleware.apply(void 0, middlewares)));
    var state$ = rxjs_1.from(store);
    var eventInput$ = new rxjs_1.Subject();
    var event$ = rxjs_1.from(eventInput$);
    var createDispatch = function (moduleName) {
        var dispatch = function (event) {
            if (completed) {
                return;
            }
            if (event == null) {
                return event;
            }
            if (isEvent_1.isEvent(event)) {
                updateMeta(moduleName, event);
                if (initializing) {
                    queue.push(event);
                    return event;
                }
                var result = store.dispatch(event);
                eventInput$.next(event);
                return result;
            }
            if (isSubscribable_1.isSubscribable(event)) {
                return new Promise(function (resolve, reject) {
                    return event.subscribe({
                        next: dispatch,
                        complete: resolve,
                        error: reject
                    });
                });
            }
            // tslint:disable-next-line strict-type-predicates
            if (typeof event === 'function') {
                return event(store.getState, dispatch);
            }
            return event;
        };
        return dispatch;
    };
    var flushQueue = function () {
        initializing = false;
        for (var _i = 0, _a = queue; _i < _a.length; _i++) {
            var event_1 = _a[_i];
            store.dispatch(event_1);
            eventInput$.next(event_1);
        }
        queue = null;
    };
    var disconnect = function () {
        eventInput$.complete();
        completed = true;
    };
    return {
        state$: state$,
        event$: event$,
        getState: store.getState,
        createDispatch: createDispatch,
        flushQueue: flushQueue,
        disconnect: disconnect
    };
};
//# sourceMappingURL=getStore.js.map