"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var constants_1 = require("../../helpers/constants");
var isPromise_1 = require("../../helpers/is/isPromise/isPromise");
var createEvent_1 = require("../createEvent/createEvent");
/**
 * @private
 */
var run = function (params, effect) {
    var response;
    try {
        response = effect(params);
    }
    catch (err) {
        return Promise.reject(err);
    }
    if (isPromise_1.isPromise(response)) {
        return response;
    }
    return Promise.resolve(response);
};
/**
 * @private
 */
var T = function () { return true; };
/**
 * Creates an effect creator. Effect is a stream, that uses provided function, and emits start, success, error and complete types.
 *
 * @param description
 * @param effect Side effect performing function, should return a Promise.
 * @param condition Function, that defines if an effect should run. Must return boolean. T by default. E.g. can be used to separate server-side effects.
 * @returns
 */
exports.createEffect = function (description, effect, condition) {
    if (condition === void 0) { condition = T; }
    var success = createEvent_1.createEvent(description + ": SUCCESS");
    var fail = createEvent_1.createEvent(description + ": FAIL");
    var start = createEvent_1.createEvent(description + ": START");
    var complete = createEvent_1.createEvent(description + ": COMPLETE");
    var _effect = effect;
    var runEffect = function (payload) {
        if (!_effect) {
            throw new Error(constants_1.APP_KEY + " error: Effect is not provided!");
        }
        if (!condition(payload)) {
            return rxjs_1.EMPTY;
        }
        return rxjs_1.concat(rxjs_1.of(start(payload)), rxjs_1.from(run(payload, _effect)
            .then(function (result) { return success(result); })
            .catch(function (error) { return fail(error); })), rxjs_1.of(complete()));
    };
    return Object.assign(runEffect, {
        start: start,
        success: success,
        fail: fail,
        complete: complete,
        getType: function () { return start.getType(); },
        use: function (fn) {
            _effect = fn;
            return runEffect;
        }
    });
};
//# sourceMappingURL=createEffect.js.map