"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var identity_1 = require("../../helpers/identity/identity");
var uniqify_1 = require("../../helpers/uniqueId/uniqify");
var createEpic_1 = require("../createEpic/createEpic");
/**
 * Checks if provided value is an instance of Error
 * @private
 */
var isError = function (a) { return a instanceof Error; };
function createEvent(description, payloadCreator, metaCreator) {
    if (payloadCreator === void 0) { payloadCreator = identity_1.identity; }
    var type = uniqify_1.uniqify(description);
    var eventCreator = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var event = {
            type: type
        };
        try {
            event.payload = isError(args[0])
                ? args[0]
                : payloadCreator.apply(void 0, args);
        }
        catch (error) {
            event.payload = error;
        }
        event.error = isError(event.payload);
        if (metaCreator) {
            event.meta = metaCreator.apply(void 0, args);
        }
        return event;
    };
    eventCreator.getType = function () { return type; };
    eventCreator.is = function (event) { return event.type === type; };
    eventCreator.epic = function (fn) {
        return createEpic_1.createEpic(eventCreator, fn);
    };
    return eventCreator;
}
exports.createEvent = createEvent;
//# sourceMappingURL=createEvent.js.map