"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var getEventType_1 = require("../../helpers/getEventType/getEventType");
var has_1 = require("../../helpers/has/has");
var isArray_1 = require("../../helpers/is/isArray/isArray");
var createEvent_1 = require("../createEvent/createEvent");
/**
 * Creates reducer with some additional methods
 * @typeparam S Reducer's state interface
 * @param initialState Initial state for the reducer
 * @param handlers Object with event types as keys and EventHandlers as values
 * @returns {Reducer}
 */
exports.createReducer = function (initialState, handlers) {
    if (handlers === void 0) { handlers = {}; }
    var innerHandlers = Object.assign({}, handlers);
    var reducer = function (state, event) {
        if (state === void 0) { state = initialState; }
        if (!event) {
            return state;
        }
        if (!has_1.has(event.type, innerHandlers)) {
            return state;
        }
        return innerHandlers[event.type](state, event.payload, event.meta);
    };
    var resetHandler = function () { return initialState; };
    reducer.has = function (eventOrType) {
        return has_1.has(getEventType_1.getEventType(eventOrType), innerHandlers);
    };
    reducer.on = function (eventOrType, handler) {
        if (isArray_1.isArray(eventOrType)) {
            eventOrType.forEach(function (event) { return reducer.on(event, handler); });
            return reducer;
        }
        innerHandlers[getEventType_1.getEventType(eventOrType)] = handler;
        return reducer;
    };
    reducer.off = function (eventOrType) {
        if (isArray_1.isArray(eventOrType)) {
            eventOrType.forEach(function (event) { return reducer.off(event); });
            return reducer;
        }
        delete innerHandlers[getEventType_1.getEventType(eventOrType)];
        return reducer;
    };
    reducer.reset = function (eventOrType) {
        if (isArray_1.isArray(eventOrType)) {
            eventOrType.forEach(function (event) { return reducer.reset(event); });
            return reducer;
        }
        innerHandlers[getEventType_1.getEventType(eventOrType)] = resetHandler;
        return reducer;
    };
    reducer.createEvents = function (model) {
        var eventNames = Object.keys(model);
        var events = {};
        eventNames.forEach(function (eventName) {
            var handler = model[eventName];
            var eventCreator = createEvent_1.createEvent("" + eventName);
            events[eventName] = eventCreator;
            reducer.on(eventCreator, handler);
        });
        return events;
    };
    return reducer;
};
//# sourceMappingURL=createReducer.js.map